# This function performs the 10-fold stratified cross-validation on the datasets.
#
# It takes the whole number 'iter', which corresponds to the iter-th line 
# of 'scenariogrid', which contains the necessary information
# on the iter-th setting.

evaluatesetting <- function(iter) {

# Extract setting information:

  dat <- scenariogrid$dat[iter]
  seed <- scenariogrid$seed[iter]
  method <- scenariogrid$method[iter]
  
  iterind <- scenariogrid$iterind[iter]
  
  
  # Load dataset:
  
  load(paste("./HornungOrdinalForests/RealDataApplications/data/", dat, sep=""))
  
  
  # Generate the folds used in the stratified cross-validation:
  
    ncv <- 10
  set.seed(seed)
  cvdiv <- makeCVdiv(data=dataset, ncv=ncv)
  
  
  # Initialize vectors of true and predicted
  # class values:
  
  ytrue <- factor(levels=levels(dataset$y))
  yhat <- factor(levels=levels(dataset$y))
  
  
  # Perform stratified cross-validation:
  
  for(i in 1:ncv) {

  # Training and test data in i-th iteration:
    datatrain <- dataset[cvdiv!=i,]
    datatest <- dataset[cvdiv==i,]
    
	# Ordered probit regression:
    if(method=="ordproblog") {
      object <- ordproblog(data=datatrain)
      predobj <- predictordproblog(object=object, newdata=datatest)
      yhattemp <- predobj$ypred
    }
    
	# Ordinal forest:
    if(method=="ordforest") {
      library("ordinalForest")
      object <- ordfor(depvar="y", data=datatrain, num.threads=2)
      predobj <- predict(object=object, newdata=datatest)
      yhattemp <- predobj$ypred
    }  
    
	# Naive ordinal forest:
    if(method=="naiveordforest") {
      library("ordinalForest")
      object <- ordfor(depvar="y", data=datatrain, naive=TRUE, num.threads=2)
      predobj <- predict(object=object, newdata=datatest)
      yhattemp <- predobj$ypred
    }  
    
	# Multi-class random forest:
    if(method=="multiclassforest") {
      library("ranger")
      object <- ranger(dependent.variable.name = "y", data = datatrain, 
                       num.trees = 5000)
      
      yhattemp <- predict(object=object, data=datatest)$predictions
    }  
    
	# True class values:
    ytrue[cvdiv==i] <- datatest$y
	# Predicted class values:
    yhat[cvdiv==i] <- yhattemp
    
  }
  
  # Combine in list:
  
  res <- list(ytrue=ytrue, yhat=yhat)
  
  # Return results:
  
  return(res)
  
}



  
# Function to generate the splittings for stratified cross-validation:

# Input parameters:

# data   - dataset
# yname  - label of the target variable
# ncv    - number of folds to use
  
makeCVdiv <- function(data, yname="y", ncv=10) {
    
  cvid <- rep(NA, length = nrow(data))
  for(i in levels(data[, yname])) cvid[data[, yname] == i] <- sample(rep(1:ncv, length = sum(data[, yname] == i)))
  
  cvid
  
}



# Wrapper function for 'polr' to perform
# ordered probit regression.

# Input parameters:

# data   - dataset, target variable has to have the label "y"

ordproblog <- function(data) {
  
  # Apply ordered probit regression:
  require("MASS")
  model <- polr(y ~ ., data = data, method = "probit")
  
  return(model)
  
}




# Wrapper function for 'predict' to perform
# prediction using a fitted order probit regression model.

# Input parameters:

# object   - object of class "polr"
# newdata  - data.frame containing the new data
#            for which prediction should be performed.

predictordproblog <- function(object, newdata) {
  
  # Predict the class values of new observations
  # using a fitted ordered probit regression model:
  require("MASS")
  yprob <- predict(object, newdata, type = "probs")
  rownames(yprob) <- 1:nrow(yprob)
  
  ypred <- factor(levels(newdata$y)[apply(yprob, 1, which.max)], levels=levels(newdata$y))
  
  return(list(ypred=ypred))
  
}
