# This function generates a pair of training dataset and test dataset,
# applies one of the three considered methods to the training dataset
# and obtains predictions on the test dataset using the prediction
# rule constructed on the training data. Moreover, among other things
# it returns the variable importance values obtained with each of the 
# three considered methods.
#
# It takes the whole number 'iter', which corresponds to the iter-th line 
# of 'scenariogrid', which contains the necessary information
# on the iter-th setting.

evaluatesetting <- function(iter) {
  
  # Extract setting information:
  
  method <- scenariogrid$method[iter]
  equalclasswidths <- scenariogrid$equalclasswidths[iter]
  numberobs <- ifelse(scenariogrid$usecorrelations_highdim_numberobs[iter] %in% c("correlations_nonhighdim_200", "nocorrelations_nonhighdim_200", "correlations_highdim"),
                      200, 400)
  highdim <- ifelse(scenariogrid$usecorrelations_highdim_numberobs[iter]=="correlations_highdim",
                    TRUE, FALSE)
  usecorrelations <- ifelse(scenariogrid$usecorrelations_highdim_numberobs[iter] %in% c("correlations_nonhighdim_200", "correlations_nonhighdim_400", "correlations_highdim"),
                            TRUE, FALSE)
  numberclass <- scenariogrid$numberclass[iter]
  simit <- scenariogrid$simit[iter]
  seed <- scenariogrid$seed[iter]
  
  
  # Set seed:
  
  set.seed(seed)
  
  
  
  # Simulation parameters (not each of them is considered in all settings):
  
  ntest <- 10000
  corstrength <- 0.8
  ncor <- 6
  nnoise <- 50
  effect11 <- 0.5
  effect12 <- 0.75
  effect13 <- 1
  sigmanoise <- 1
  
  
  # Simulate a pair of training data and test data:
  
  traintestdata <- simtraintestdata(equalclasswidths=equalclasswidths, J=numberclass, ntrain=numberobs, 
                                    ntest=ntest, correlations=usecorrelations, corstrength=corstrength, 
                                    ncor=ncor, nnoise=nnoise, effect11=effect11, effect12=effect12, 
									effect13=effect13, sigmanoise=sigmanoise, highdim=highdim)

  datatrain <- traintestdata$datatrain
  datatest <- traintestdata$datatest


   # Multi-class random forest:
  
  if(method=="multiclassforest") {
    
	# Construct multi-class random forest:
    require("ranger")
    object <- ranger(dependent.variable.name = "y", data = datatrain, 
                     num.trees = 5000, importance="permutation")
    
	# Obtain predictions:
    preds <- predict(object=object, data=datatest)$predictions
    preds <- as.numeric(as.character(preds))
	# Sometimes there are values of 'NA' with this method. If so, replace the NA value
	# by a random class value, where the sampling probabilities for the individual classes
	# are proportional to the class frequencies in the training data:
    preds[is.na(preds)] <- sample(1:numberclass, prob=table(datatrain$y)/sum(table(datatrain$y)), size=length(preds[is.na(preds)]), replace=TRUE)
    yhattemp <- factor(preds, levels=1:numberclass)
    
	# Results: true class values, predicted class values, variable importance values,
	# true partition of [0,1]:
    res <- list(ytrue=datatest$y, yhat=yhattemp, varimp=object$variable.importance, 
	borderstrue=traintestdata$quborders, bordershat=NA)
    
  }
  
  
  # Ordinal forest:
  
  if(method=="ordforest") {
    
	# Construct ordinal forest:
    require("ordinalForest")
    ordforobj <- ordfor(depvar="y", data=datatrain, num.threads=2)
	
	# Obtain predictions:
    preds <- predict(ordforobj, newdata=datatest)$ypred
    
	# Results: true class values, predicted class values, variable importance values,
	# true partition of [0,1], estimated partition of [0,1]:
    res <- list(ytrue=datatest$y, yhat=preds, varimp=ordforobj$varimp, borderstrue=traintestdata$quborders, bordershat=ordforobj$bordersbest)
    
  }  
  
  
  # Naive ordinal forest:
  
  if(method=="naiveordforest") {
    
	# Construct naive ordinal forest:
    require("ordinalForest")
    ordforobj <- ordfor(depvar="y", data=datatrain, naive=TRUE, num.threads=2)
		
	# Obtain predictions:
    preds <- predict(ordforobj, newdata=datatest)$ypred
        
	# Results: true class values, predicted class values, variable importance values,
	# true partition of [0,1]:
    res <- list(ytrue=datatest$y, yhat=preds, varimp=ordforobj$varimp, borderstrue=traintestdata$quborders, bordershat=NA)
    
  }  
  
  # Return results:
  return(res)
  
}






# This function generates a pair of training dataset and test dataset,
# applies ordinal forest to the training dataset, where one of the three
# considered variants of the performance functions is used,
# and obtains predictions on the test dataset using the prediction
# rule constructed on the training data.
#
# It takes the whole number 'iter', which corresponds to the iter-th line 
# of 'scenariogrid', which contains the necessary information
# on the iter-th setting.

evaluatesettingperffunct <- function(iter) {
  
# Extract setting information:

  equalclasswidths <- scenariogrid$equalclasswidths[iter]
  numberobs <- ifelse(scenariogrid$usecorrelations_highdim_numberobs[iter] %in% c("correlations_nonhighdim_200", "nocorrelations_nonhighdim_200", "correlations_highdim"),
                      200, 400)
  highdim <- ifelse(scenariogrid$usecorrelations_highdim_numberobs[iter]=="correlations_highdim",
                    TRUE, FALSE)
  usecorrelations <- ifelse(scenariogrid$usecorrelations_highdim_numberobs[iter] %in% c("correlations_nonhighdim_200", "correlations_nonhighdim_400", "correlations_highdim"),
                            TRUE, FALSE)
  numberclass <- scenariogrid$numberclass[iter]
  simit <- scenariogrid$simit[iter]
  seed <- scenariogrid$seed[iter]
  performancefunction <- scenariogrid$performancefunction[iter]
	
		
# Set seed:	

  set.seed(seed)
      
  
  # Simulation parameters (not each of them is considered in all settings):
  
  ntest <- 10000
  corstrength <- 0.8
  ncor <- 6
  nnoise <- 50
  effect11 <- 0.5
  effect12 <- 0.75
  effect13 <- 1
  sigmanoise <- 1
  
  
  # Simulate a pair of training data and test data:
  
  traintestdata <- simtraintestdata(equalclasswidths=equalclasswidths, J=numberclass, ntrain=numberobs, 
                                    ntest=ntest, correlations=usecorrelations, corstrength=corstrength, 
                                    ncor=ncor, nnoise=nnoise, effect11=effect11, effect12=effect12, effect13=effect13, sigmanoise=sigmanoise, highdim=highdim)
  
  datatrain <- traintestdata$datatrain
  datatest <- traintestdata$datatest

  
  # Apply ordinal forest:
    
    require("ordinalForest")
    
	# If the performance measure "oneclass" is used, then using the 
	# argument "classimp" the category to be prioritized has to 
	# be provided:
	if(performancefunction!="oneclass")
    ordforobj <- ordfor(depvar="y", data=datatrain, num.threads=2, perffunction=performancefunction)
	else
	ordforobj <- ordfor(depvar="y", data=datatrain, num.threads=2, perffunction="oneclass", classimp="2")
	
	# Obtain predictions:
    preds <- predict(ordforobj, newdata=datatest)$ypred
    
	# Results: true class values, predicted class values:
    res <- list(ytrue=datatest$y, yhat=preds)
   
   # Return results:
  return(res)
  
}







# This function generates a pair of training dataset and test dataset,
# applies ordinal forest to the training dataset, where a particular 
# value for one (combination) of the hyperparameters is used (depending on setting)
# using the default values for all other hyperparameters,
# and obtains predictions on the test dataset using the prediction
# rule constructed on the training data.
#
# It takes the whole number 'iter', which corresponds to the iter-th line 
# of 'scenariogrid', which contains the necessary information
# on the iter-th setting.

evaluatesettinghyperparam <- function(iter) {
  
# Extract setting information:

  paramsetting <- scenariogrid$paramsetting[iter]
  simsetting <- scenariogrid$simsetting[iter]
  datasetind <- scenariogrid$datasetind[iter]
  seed <- scenariogrid$seed[iter]
  
  
  # Set seed:
  
  set.seed(seed)
  
  
  # Generate pair of training data and test data 
  # using the low-dimensional setting:
  
  if(simsetting=="lowdim") {
    
    equalclasswidths <- TRUE
    numberobs <- 200
    highdim <- FALSE
    usecorrelations <- TRUE
    numberclass <- 6
    
    ntest <- 10000
    corstrength <- 0.8
    ncor <- 6
    nnoise <- 50
    effect11 <- 0.5
    effect12 <- 0.75
    effect13 <- 1
    sigmanoise <- 1
    
    traintestdata <- simtraintestdata(equalclasswidths=equalclasswidths, J=numberclass, ntrain=numberobs, 
                                      ntest=ntest, correlations=usecorrelations, corstrength=corstrength, 
                                      ncor=ncor, nnoise=nnoise, effect11=effect11, effect12=effect12, effect13=effect13, sigmanoise=sigmanoise, highdim=highdim)
    
  }
  
  
  # Generate pair of training data and test data 
  # using the high-dimensional setting:
  
  if(simsetting=="highdim") {
    
    equalclasswidths <- TRUE
    numberobs <- 200
    highdim <- TRUE
    usecorrelations <- TRUE
    numberclass <- 6
    
    ntest <- 10000
    corstrength <- 0.8
    ncor <- 6
    nnoise <- 50
    effect11 <- 0.5
    effect12 <- 0.75
    effect13 <- 1
    sigmanoise <- 1
    
    traintestdata <- simtraintestdata(equalclasswidths=equalclasswidths, J=numberclass, ntrain=numberobs, 
                                      ntest=ntest, correlations=usecorrelations, corstrength=corstrength, 
                                      ncor=ncor, nnoise=nnoise, effect11=effect11, effect12=effect12, effect13=effect13, sigmanoise=sigmanoise, highdim=highdim)
    
  }
  
  datatrain <- traintestdata$datatrain
  datatest <- traintestdata$datatest
  
  
  
  # Apply ordinal forest where a particular 
  # value for one (combination) of the hyperparameters is used (depending on setting)
  # using the default values for all other hyperparameters:
  
  require("ordinalForest")
  
  if(length(grep("npermtrial", paramsetting)) > 0)
    eval(parse(text=paste("ordforobj <- ordfor(depvar=\"y\", data=datatrain, num.threads=2, permperdefault = TRUE, ", paramsetting, ")", sep="")))
  else
    eval(parse(text=paste("ordforobj <- ordfor(depvar=\"y\", data=datatrain, num.threads=2, ", paramsetting, ")", sep="")))
  
  
  # Obtain predictions:
  preds <- predict(ordforobj, newdata=datatest)$ypred
 
 
 # Results: true class values, predicted class values:
  res <- list(ytrue=datatest$y, yhat=preds)
  
  # Return results:
  return(res)
  
}







# Functions used to generate the simulated data:




# This function is called by 'simdata' and generates the covariate values
# for a dataset in the simulation. The code is strongly based on the code
# provided by Janitza et al. (2016) in their electronic appendix.

# S Janitza, G Tutz, and A-L Boulesteix. 
# Random forest for ordinal responses: prediction and variable selection.
# Computational Statistics and Data Analysis, 96:57-73, 2016.

# Input parameters:

# n             - number of observations
# correlations  - boolean. should the covariates be correlated?
# corstrength   - strength of correlation between correlated influential covariates
# ncor          - number of correlated influential covariates
# nnoise        - number of non-influential (noise) covariates
#                 in the non-high-dimensional scenario
# highdim       - boolean. Should the covariate vector be high-dimensional?

simpreds <- function(n, correlations, corstrength, 
                     ncor, nnoise, highdim) {
  
  require("mvtnorm")
  
  if(correlations == TRUE){ # for settings with correlations between predictors
    
    if(!highdim) {
      
      # covariance matrix
      signal_sigma08 <- matrix(corstrength, nrow = ncor, ncol = ncor)  # 6 correlated signal predictors
      noise_sigma08  <- matrix(0.8, nrow = 10, ncol = 10)
      noise_sigma06  <- matrix(0.6, nrow = 10, ncol = 10)
      noise_sigma04  <- matrix(0.4, nrow = 10, ncol = 10)
      noise_sigma02  <- matrix(0.2, nrow = 10, ncol = 10)
      
      diag(signal_sigma08) <- diag(noise_sigma08) <- diag(noise_sigma06) <- diag(noise_sigma04) <- diag(noise_sigma02) <- 1
      
      # generate correlated predictors
      X_cor_signal   <- rmvnorm(n = n, sigma = signal_sigma08) 
      X_cor_noise08  <- rmvnorm(n = n, sigma = noise_sigma08) 
      X_cor_noise06  <- rmvnorm(n = n, sigma = noise_sigma06) 
      X_cor_noise04  <- rmvnorm(n = n, sigma = noise_sigma04)
      X_cor_noise02  <- rmvnorm(n = n, sigma = noise_sigma02)
      
      
      # generate uncorrelated predictors
      
      X_uncor_noise  <- rmvnorm(n = n, mean = rep(0, nnoise - 40))
      X_uncor_signal <- rmvnorm(n = n, mean = rep(0, 15 - ncor)) # 9 uncor. signal pred
      
      # combine predictors in a matrix
      
      X <- cbind(X_cor_signal[,1], X_uncor_signal[,1], X_cor_signal[,2], X_uncor_signal[,c(2, 3)],
                 X_cor_signal[,3], X_uncor_signal[,4], X_cor_signal[,4], X_uncor_signal[,c(5, 6)],
                 X_cor_signal[,5], X_uncor_signal[,7], X_cor_signal[,6], X_uncor_signal[,c(8, 9)],
                 X_cor_noise08, X_cor_noise06, X_cor_noise04, X_cor_noise02, X_uncor_noise)
      
    }
    else {
      
      # covariance matrix
      signal_sigma08 <- matrix(corstrength, nrow = ncor, ncol = ncor)  # 6 correlated signal predictors
      noise_sigma08  <- matrix(0.8, nrow = 10, ncol = 10)
      noise_sigma06  <- matrix(0.6, nrow = 10, ncol = 10)
      noise_sigma04  <- matrix(0.4, nrow = 10, ncol = 10)
      noise_sigma02  <- matrix(0.2, nrow = 10, ncol = 10)
      
      diag(signal_sigma08) <- diag(noise_sigma08) <- diag(noise_sigma06) <- diag(noise_sigma04) <- diag(noise_sigma02) <- 1
      
      # generate correlated predictors
      X_cor_signal   <- rmvnorm(n = n, sigma = signal_sigma08)
      
      
      X_cor_noise08  <- list()
      for(i in 1:20)
        X_cor_noise08[[i]] <- rmvnorm(n = n, sigma = noise_sigma08)
      X_cor_noise08 <- do.call(cbind, X_cor_noise08)
      
      X_cor_noise06  <- list()
      for(i in 1:20)
        X_cor_noise06[[i]] <- rmvnorm(n = n, sigma = noise_sigma06)
      X_cor_noise06 <- do.call(cbind, X_cor_noise06)
      
      X_cor_noise04  <- list()
      for(i in 1:20)
        X_cor_noise04[[i]] <- rmvnorm(n = n, sigma = noise_sigma04)
      X_cor_noise04 <- do.call(cbind, X_cor_noise04)
      
      X_cor_noise02  <- list()
      for(i in 1:20)
        X_cor_noise02[[i]] <- rmvnorm(n = n, sigma = noise_sigma02) 
      X_cor_noise02 <- do.call(cbind, X_cor_noise02)
      
      # generate uncorrelated predictors
      
      X_uncor_noise  <- list()
      for(i in 1:20)
        X_uncor_noise[[i]] <- rmvnorm(n = n, mean = rep(0, 10))
      X_uncor_noise <- do.call(cbind, X_uncor_noise)
      
      
      X_uncor_signal <- rmvnorm(n = n, mean = rep(0, 15 - ncor)) # 9 uncor. signal pred
      
      # combine predictors in a matrix
      
      X <- cbind(X_cor_signal[,1], X_uncor_signal[,1], X_cor_signal[,2], X_uncor_signal[,c(2, 3)],
                 X_cor_signal[,3], X_uncor_signal[,4], X_cor_signal[,4], X_uncor_signal[,c(5, 6)],
                 X_cor_signal[,5], X_uncor_signal[,7], X_cor_signal[,6], X_uncor_signal[,c(8, 9)],
                 X_cor_noise08, X_cor_noise06, X_cor_noise04, X_cor_noise02, X_uncor_noise)
      
    }
    
    
  }else{ # for settings with uncorrelated predictors only
    X <- matrix(rnorm(n = n * (nnoise + 15)), nrow = n)
  }
  
  return(X)
  
}




# This function is called by 'simdata' and generates the values of the linear
# predictor for a dataset in the simulation. The code is strongly based on 
# the code provided by Janitza et al. (2016) in their electronic appendix.

# S Janitza, G Tutz, and A-L Boulesteix. 
# Random forest for ordinal responses: prediction and variable selection.
# Computational Statistics and Data Analysis, 96:57-73, 2016.

# Input parameters:

# X         - covariate matrix (generated by 'simpreds')
# nnoise    - number of non-influential (noise) covariates
#             in the non-high-dimensional scenario
# effect11  - effect size of the first five influential
#             covariates
# effect12  - effect size of the sixth to the tenth influential
#             covariate
# effect13  - effect size of the eleventh to the fifteenth influential
#             covariate

simlinpred <- function(X, nnoise, effect11, effect12, effect13) {
  
  if(ncol(X)!=1015)
    beta <- rep(c(effect13, effect12, effect11, 0), c(5, 5, 5, nnoise))
  else
    beta <- rep(c(effect13, effect12, effect11, 0), c(5, 5, 5, 1000))
  
  linpred <- (X%*%beta)[,1]
  
  return(linpred)
  
}




# This function is called by 'simdata' and generates the values of the latent
# continuous target variable for a dataset in the simulation.

# Input parameters:

# linpred     - values of the linear predictor (generated by 'simlinpred')
# sigmanoise  - standard deviation of residuals added to the linear predictor

simynum <- function(linpred, sigmanoise) {
  
  linpred + rnorm(n=length(linpred), sd=sigmanoise)
  
}





# This function is called by 'simtraintestdata' and generates a dataset in the simulation.

# Input parameters:

# equalclasswidths  - boolean. Should equal class widths be used?
# borders           - interval boundaries used to assign
#                     the values of the latent continuous target
#                     variable to one of the J classes
# n                 - number of observations
# correlations      - boolean. should the covariates be correlated?
# corstrength       - strength of correlation between correlated influential covariates
# ncor              - number of correlated influential covariates
# nnoise            - number of non-influential (noise) covariates
#                     in the non-high-dimensional scenario
# effect11          - effect size of the first five influential
#                     covariates
# effect12          - effect size of the sixth to the tenth influential
#                     covariate
# effect13          - effect size of the eleventh to the fifteenth influential
#                     covariate
# sigmanoise        - standard deviation of residuals added to the linear predictor
# highdim           - boolean. Should the covariate vector be high-dimensional?

simdata <- function(equalclasswidths, borders, n, correlations, corstrength, ncor, nnoise, 
                    effect11, effect12, effect13, sigmanoise, highdim) {
  
  J <- length(borders) - 1
  
  beta <- rep(c(effect13, effect12, effect11, 0), c(5, 5, 5, nnoise))
  
  if(!equalclasswidths)
    X <- simpreds(n = n, correlations=correlations, corstrength=corstrength, ncor=ncor, nnoise=nnoise, highdim=highdim)
  else
    X <- simpreds(n = n*1.1, correlations=correlations, corstrength=corstrength, ncor=ncor, nnoise=nnoise, highdim=highdim)
  
  
  ynum <- simynum(linpred=simlinpred(X=X, nnoise=nnoise, effect11=effect11, effect12=effect12, effect13=effect13), sigmanoise=sigmanoise)
  
  if(equalclasswidths) {
  if(!highdim) {
  if(correlations)
    includeind <- sample(which((ynum >= qufuncor(0.001)) & (ynum <= qufuncor(1-0.001))), size=n)
if(!correlations)	
    includeind <- sample(which((ynum >= qufununcor(0.001)) & (ynum <= qufununcor(1-0.001))), size=n)
}
else
	    includeind <- sample(which((ynum >= qufunhighdim(0.001)) & (ynum <= qufunhighdim(1-0.001))), size=n)
    X <- X[includeind,]
    ynum <- ynum[includeind]
  }
  
  y <- factor(sapply(ynum, function(x) max(which(x >= borders))), levels=1:J)
  
  dataset <- data.frame(X, y)
  
  return(dataset)
  
}





# This function generates a pair of training dataset and test 
# dataset in the simulation.

# Input parameters:

# equalclasswidths  - boolean. Should equal class widths be used?
# J                 - number of classes of the ordinal target variable
# ntrain            - number of observations in the training data
# ntest             - number of observations in the test data
# correlations      - boolean. should the covariates be correlated?
# corstrength       - strength of correlation between correlated influential covariates
# ncor              - number of correlated influential covariates
# nnoise            - number of non-influential (noise) covariates
#                     in the non-high-dimensional scenario
# effect11          - effect size of the first five influential
#                     covariates
# effect12          - effect size of the sixth to the tenth influential
#                     covariate
# effect13          - effect size of the eleventh to the fifteenth influential
#                     covariate
# sigmanoise        - standard deviation of residuals added to the linear predictor
# highdim           - boolean. Should the covariate vector be high-dimensional?


simtraintestdata <- function(equalclasswidths, J, ntrain, ntest, correlations, corstrength, 
                             ncor, nnoise, effect11, effect12, effect13, sigmanoise, highdim) {
  
  if(!equalclasswidths) {
    quborders <- sort(runif(J-1))
    if(!highdim) {
      if(correlations)
        borders <- c(-Inf, qufuncor(quborders), Inf)
      if(!correlations)
        borders <- c(-Inf, qufununcor(quborders), Inf)
     quborders <- c(0, quborders, 1)
    }
    else
      borders <- c(-Inf, qufunhighdim(quborders), Inf)
  }
  else {
    quborders <- NA
    if(!highdim) {
      if(correlations)
        borders <- seq(qufuncor(0.001), qufuncor(1-0.001), length=J+1)
      if(!correlations) 
        borders <- seq(qufununcor(0.001), qufununcor(1-0.001), length=J+1)					
    }
    else
      borders <- seq(qufunhighdim(0.001), qufunhighdim(1-0.001), length=J+1)  
  }
  
  datatrain <- simdata(equalclasswidths=equalclasswidths, borders=borders, n=ntrain, correlations=correlations, 
                       corstrength=corstrength, ncor=ncor, nnoise=nnoise, effect11=effect11, 
                       effect12=effect12, effect13=effect13, sigmanoise=sigmanoise, highdim=highdim)
  
  datatest <- simdata(equalclasswidths=equalclasswidths, borders=borders, n=ntest, correlations=correlations, 
                      corstrength=corstrength, ncor=ncor, nnoise=nnoise, effect11=effect11, 
                      effect12=effect12, effect13=effect13, sigmanoise=sigmanoise, highdim=highdim)
  
  return(list(datatrain=datatrain, datatest=datatest, borders=borders, quborders=quborders))
  
}    
