# In case the folder "HornungOrdinalForests" is not contained in the
# current R working directory, use the following line and replace
# *directory that contains the folder 'HornungOrdinalForests'* by
# the path to the directory that contains the folder
# "HornungOrdinalForests":

# setwd("*directory that contains the folder 'HornungOrdinalForests'*")


# Load and process results:
############################


# Load results:

load("./HornungOrdinalForests/Results/RealDataAnalysis_Results.Rda")



# Make scenario table to match the results to the corresponing meta information:

# Make table of settings:

method <- c("ordforest", "naiveordforest", "multiclassforest", "ordproblog")
dat <- c("mammography_dataset.Rda", "supportstudy_dataset.Rda", 
         "vlbw_dataset.Rda", "whinequality_dataset.Rda", "nhanes_dataset.Rda")
cvind <- 1:10

scenariogrid <- expand.grid(cvind=cvind, method=method, dat=dat, stringsAsFactors = FALSE)
scenariogrid <- scenariogrid[,ncol(scenariogrid):1]

set.seed(1234)
seeds <- sample(1000:10000000, size=length(cvind))

scenariogrid$seed <- rep(seeds, times=length(method)*length(dat))

dim(scenariogrid)


# Randomly permute rows of the table containing the settings.
# This is performed to ensure a comparable computational burdens for
# the jobs to be performed in parallel:

set.seed(1234)
reorderind <- sample(1:nrow(scenariogrid))
scenariogrid <- scenariogrid[reorderind,]





# One cross-validation iteration resulted in an error for "winequality"
# for ordered probit regression:

scenariogrid[sapply(Results, class)=="try-error",]
Results[sapply(Results, class)=="try-error"]



# Exclude this iteration:

availres <- sapply(Results, class)!="try-error"
Results <- Results[availres]
scenariogrid <- scenariogrid[availres,]



# Calculate the (weighted) Kappa values:

library("psych")

kappasquadratic <- sapply(Results, function(y) {
  
  x <- data.frame(ytrue=y$ytrue, yhat=y$yhat)
  
  J <- length(unique(c(x$ytrue, x$yhat)))
  
  myw <- matrix(0, ncol = J, nrow = J)
  myw[] <- abs((col(myw) - row(myw)))^2
  myw <- 1 - myw/(J - 1)^2
  
  resi <- try(cohen.kappa(x, w=myw)$weighted.kappa)
  if(class(resi)=="try-error")
    resi <- NA
  resi})


kappaslinear <- sapply(Results, function(y) {
  
  x <- data.frame(ytrue=y$ytrue, yhat=y$yhat)
  
  J <- length(unique(c(x$ytrue, x$yhat)))
  
  myw <- matrix(0, ncol = J, nrow = J)
  myw[] <- abs((col(myw) - row(myw)))
  myw <- 1 - myw/(J - 1)
  
  resi <- try(cohen.kappa(x, w=myw)$weighted.kappa)
  if(class(resi)=="try-error")
    resi <- NA
  resi})


kappas <- sapply(Results, function(y) {
  
  x <- data.frame(ytrue=y$ytrue, yhat=y$yhat)
  
  resi <- try(cohen.kappa(x)$kappa)
  if(class(resi)=="try-error")
    resi <- NA
  resi})





# Combine results in a data.frame:

results <- scenariogrid
results$kappaqu <- kappasquadratic
results$kappalin <- kappaslinear
results$kappa <- kappas



# Order rows in the table of results:

results <- results[order(results$cvind, results$dat, results$method),]




# Mean Kappa values for each combination of dataset and method:
###############################################################

library("plyr")
resultsum <- ddply(results, .variables=c("dat", "method"), .fun=summarise, kappaqu=mean(kappaqu),
                   kappalin=mean(kappalin), kappa=mean(kappa))
resultsum




# Boxplots of (weighted) Kappa values:
######################################

results$dat <- gsub("whine", "wine", gsub("_dataset.Rda", "", results$dat))
results$method[results$method=="multiclassforest"] <- "multi-class RF"
results$method[results$method=="ordforest"] <- "OF"
results$method[results$method=="naiveordforest"] <- "naive OF"
results$method[results$method=="ordproblog"] <- "ordered probit"

library("ggplot2")

p <- ggplot(data=results, aes(x=method, y=kappaqu)) + facet_wrap(~dat, nrow=2, scales = 'free_y') + 
  geom_boxplot() + theme_bw() + ylab("Quadratically weighted Kappa") + 
  theme(axis.text.x=element_text(size=16, angle=53, vjust=1, hjust=1, colour="black"),
        axis.text.y=element_text(size=14, colour="black"), axis.title.x=element_blank(),
        strip.text.x = element_text(size = 16),
        axis.title=element_text(size=16)) +
  scale_x_discrete(labels=c("multi-class RF", "naive OF", "OF", "ordered probit\nregression")) + theme(plot.margin=unit(c(1,1,-2,1), "mm"))

p

ggsave("./HornungOrdinalForests/Results/Figures/FigureRealKappaquad.pdf", width=10, height=6)


p <- ggplot(data=results, aes(x=method, y=kappalin)) + facet_wrap(~dat, nrow=2, scales = 'free_y') + 
  geom_boxplot() + theme_bw() + ylab("Linearly weighted Kappa") + 
  theme(axis.text.x=element_text(size=16, angle=53, vjust=1, hjust=1, colour="black"),
        axis.text.y=element_text(size=14, colour="black"), axis.title.x=element_blank(),
        strip.text.x = element_text(size = 16),
        axis.title=element_text(size=16)) +
  scale_x_discrete(labels=c("multi-class RF", "naive OF", "OF", "ordered probit\nregression")) + theme(plot.margin=unit(c(1,1,-2,1), "mm"))

p

ggsave("./HornungOrdinalForests/Results/Figures/FigureRealKappalinear.pdf", width=10, height=6)


p <- ggplot(data=results, aes(x=method, y=kappa)) + facet_wrap(~dat, nrow=2, scales = 'free_y') + 
  geom_boxplot() + theme_bw() + ylab("Cohen's Kappa (unweighted)") + 
  theme(axis.text.x=element_text(size=16, angle=53, vjust=1, hjust=1, colour="black"),
        axis.text.y=element_text(size=14, colour="black"), axis.title.x=element_blank(),
        strip.text.x = element_text(size = 16),
        axis.title=element_text(size=16)) +
  scale_x_discrete(labels=c("multi-class RF", "naive OF", "OF", "ordered probit\nregression")) + theme(plot.margin=unit(c(1,1,-2,1), "mm"))

p

ggsave("./HornungOrdinalForests/Results/Figures/FigureRealKappa.pdf", width=10, height=6)
