# In case the folder "HornungOrdinalForests" is not contained in the
# current R working directory, use the following line and replace
# *directory that contains the folder 'HornungOrdinalForests'* by
# the path to the directory that contains the folder
# "HornungOrdinalForests":

# setwd("*directory that contains the folder 'HornungOrdinalForests'*")


# Load and process results:
############################

# The R script "Simulation.R" (subfolder "Simulation") produces an
# Rda file labeled "Simulation_Results.Rda". This file
# is really large (93 MB), because it contains the
# true and predicted class values for each combination of test dataset
# and method and the variable importance vector obtained for each
# training dataset and method.
# Therefore, this Rda file is not included in the electronic
# appendix (it can, however, be obtained using "Simulation.R"),
# but instead a much smaller Rda file "Simulation_Results_WithKappa.Rda"
# is included that contains the calculated (weighted) Kappa values instead
# of the true and predicted class values and the AUC values calculated for
# the variable importances instead of the variable importance vectors.
# This Rda file "Simulation_Results_WithKappa.Rda" is producable using
# the following code within "if(FALSE) {....}" (for this code, of course, 
# "Simulation_Results.Rda" is required):  

if(FALSE) {
  
  # Load simulation results:
  
  load("./HornungOrdinalForests/Results/Simulation_Results.Rda")
  
  # Calculate different versions of Kappa:
  
  unweightedkappa <- function(ytrue, yhat) {
    
    require("psych")
    
    x <- data.frame(ytrue=ytrue, yhat=yhat)
    
    cohen.kappa(x)$kappa
    
  }
  
  linearkappa <- function(ytrue, yhat) {
    
    require("psych")
    
    x <- data.frame(ytrue=ytrue, yhat=yhat)
    
    J <- length(unique(c(x$ytrue, x$yhat)))
    
    myw <- matrix(0, ncol = J, nrow = J)
    myw[] <- abs((col(myw) - row(myw)))
    myw <- 1 - myw/(J - 1)
    
    cohen.kappa(x, w=myw)$weighted.kappa
    
  }
  
  quadratickappa <- function(ytrue, yhat) {
    
    require("psych")
    
    x <- data.frame(ytrue=ytrue, yhat=yhat)
    
    J <- length(unique(c(x$ytrue, x$yhat)))
    
    myw <- matrix(0, ncol = J, nrow = J)
    myw[] <- abs((col(myw) - row(myw)))^2
    myw <- 1 - myw/(J - 1)^2
    
    cohen.kappa(x, w=myw)$weighted.kappa
    
  }
  
  kappaqu <- sapply(Results, function(x) quadratickappa(x$ytrue, x$yhat))
  kappalin <- sapply(Results, function(x) linearkappa(x$ytrue, x$yhat))
  cohenskappa <- sapply(Results, function(x) unweightedkappa(x$ytrue, x$yhat))
  
  Results <- mapply(function(x, y) c(x, 'kappaqu'=y), Results, kappaqu, SIMPLIFY=FALSE)
  Results <- mapply(function(x, y) c(x, 'kappalin'=y), Results, kappalin, SIMPLIFY=FALSE)
  Results <- mapply(function(x, y) c(x, 'kappa'=y), Results, cohenskappa, SIMPLIFY=FALSE)
  Results <- lapply(Results, function(x) {
    x$ytrue <- NULL
    x$yhat <- NULL
    return(x)
  })
  
  
  # Compute AUC values of variable importances:
  
  computeAUC <- function(z) {
    
    require("ROCR")
    
    effect_ind <- rep(0, length(z))
    effect_ind[1:15] <- 1
    
    unlist(performance(prediction(z, effect_ind), measure = "auc")@y.values)
    
  }
  
  aucs <- sapply(Results, function(x) computeAUC(x$varimp))
  
  Results <- mapply(function(x, y) c(x, 'auc'=y), Results, aucs, SIMPLIFY=FALSE)
  
  Results <- lapply(Results, function(x) {
    x$varimp <- NULL
    return(x)
  })
  
  
  # Save processed results:
  
  save(Results, file="./HornungOrdinalForests/Results/Simulation_Results_WithKappa.Rda")
  
}

# Load the processed results:

load("./HornungOrdinalForests/Results/Simulation_Results_WithKappa.Rda")





# Make scenario table to match the results to the corresponding meta information:

usecorrelations_highdim_numberobs <- c("correlations_nonhighdim_200", "nocorrelations_nonhighdim_200", 
                                       "correlations_nonhighdim_400", "nocorrelations_nonhighdim_400", 
                                       "correlations_highdim")
equalclasswidths <- c(TRUE, FALSE)
numberclass <- c(3,6,9)

method <- c("ordforest", "naiveordforest", "multiclassforest")

simit <- 1:100


scenariogrid <- expand.grid(method=method, simit=simit, equalclasswidths=equalclasswidths,  
                            numberclass=numberclass, usecorrelations_highdim_numberobs=usecorrelations_highdim_numberobs, stringsAsFactors = FALSE)
scenariogrid <- scenariogrid[,ncol(scenariogrid):1]



set.seed(1234)
seeds <- sample(1000:10000000, size=length(simit)*length(equalclasswidths)*length(numberclass)*length(usecorrelations_highdim_numberobs))

scenariogrid$seed <- rep(seeds, each=length(method))


set.seed(1234)
reorderind <- sample(1:nrow(scenariogrid))
scenariogrid <- scenariogrid[reorderind,]





# Make results table:

results <- scenariogrid

tempvar <- results$usecorrelations_highdim_numberobs

results$correlation <- TRUE
results$correlation[grep("nocorrelations", tempvar)] <- FALSE

results$highdim <- TRUE
results$highdim[grep("nonhighdim", tempvar)] <- FALSE

results$n <- 200
results$n[grep("400", tempvar)] <- 400


results$kappaqu <- sapply(Results, function(x) x$kappaqu)
results$kappalin <- sapply(Results, function(x) x$kappalin)
results$kappa <- sapply(Results, function(x) x$kappa)

results$auc <- sapply(Results, function(x) x$auc)




# Assign shorter or more adequate labels to setting parameter levels:

names(results)[names(results)=="usecorrelations_highdim_numberobs"] <- "covariates_n"
results$covariates_n[results$covariates_n=="correlations_nonhighdim_200"] <- "correlated_n200"
results$covariates_n[results$covariates_n=="nocorrelations_nonhighdim_200"] <- "independent_n200"
results$covariates_n[results$covariates_n=="correlations_nonhighdim_400"] <- "correlated_n400"
results$covariates_n[results$covariates_n=="nocorrelations_nonhighdim_400"] <- "independent_n400"
results$covariates_n[results$covariates_n=="correlations_highdim"] <- "highdim"

results$covariates_n <- factor(results$covariates_n, levels=c("correlated_n200", "correlated_n400", "independent_n200", "independent_n400", "highdim"))

names(results)[names(results)=="equalclasswidths"] <- "intervals"
results$intervals[results$intervals=="TRUE"] <- "equal width"
results$intervals[results$intervals=="FALSE"] <- "random width"

names(results)[names(results)=="numberclass"] <- "nclass"
results$nclass[results$nclass=="3"] <- "nclass = 3"
results$nclass[results$nclass=="6"] <- "nclass = 6"
results$nclass[results$nclass=="9"] <- "nclass = 9"

results$method[results$method=="multiclassforest"] <- "multi-class RF"
results$method[results$method=="ordforest"] <- "OF"
results$method[results$method=="naiveordforest"] <- "naive OF"



# In case of the settings with both high-dimensional data and intervals of random width
# the true interval borders were stored wrongly (the borders 0 and 1 were missing).
# ---> Add these to the borders:

relind <- which(results$covariates_n=="highdim" & results$intervals=="random width")
for(i in seq(along=relind)) {
  Results[[relind[i]]]$borderstrue <- c(0, Results[[relind[i]]]$borderstrue, 1)
}







# Boxplots of Kappa values for all settings:
#############################################

library("ggplot2")

p <- ggplot(data=results[results$intervals=="equal width",], aes(x=method, y=kappalin)) + facet_wrap(~ covariates_n + nclass, ncol=3, scales="free_y") + 
  geom_boxplot(width = 0.7) + theme_bw() + ylab("Linearly weighted Kappa") + 
  theme(axis.text.x=element_text(size=16, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=14, colour="black"),
        axis.title.x=element_blank(),
        axis.title.y=element_text(size=16),
        strip.text.x = element_text(size = 16))
p

ggsave("./HornungOrdinalForests/Results/Figures/FigureSimulationEqualWidthKappalinear.pdf", width=9*1.2, height=10*1.2)


p <- ggplot(data=results[results$intervals=="random width",], aes(x=method, y=kappalin)) + facet_wrap(~ covariates_n + nclass, ncol=3, scales="free_y") + 
  geom_boxplot(width = 0.7) + theme_bw() + ylab("Linearly weighted Kappa") + 
  theme(axis.text.x=element_text(size=16, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=14, colour="black"),
        axis.title.x=element_blank(),
        axis.title.y=element_text(size=16),
        strip.text.x = element_text(size = 16))
p

ggsave("./HornungOrdinalForests/Results/Figures/FigureSimulationRandomWidthKappalinear.pdf", width=9*1.2, height=10*1.2)


p <- ggplot(data=results[results$intervals=="equal width",], aes(x=method, y=kappa)) + facet_wrap(~ covariates_n + nclass, ncol=3, scales="free_y") + 
  geom_boxplot(width = 0.7) + theme_bw() + ylab("Cohen's Kappa (unweighted)") + 
  theme(axis.text.x=element_text(size=16, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=14, colour="black"),
        axis.title.x=element_blank(),
        axis.title.y=element_text(size=16),
        strip.text.x = element_text(size = 16))
p

ggsave("./HornungOrdinalForests/Results/Figures/FigureSimulationEqualWidthKappa.pdf", width=9*1.2, height=10*1.2)


p <- ggplot(data=results[results$intervals=="random width",], aes(x=method, y=kappa)) + facet_wrap(~ covariates_n + nclass, ncol=3, scales="free_y") + 
  geom_boxplot(width = 0.7) + theme_bw() + ylab("Cohen's Kappa (unweighted)") + 
  theme(axis.text.x=element_text(size=16, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=14, colour="black"),
        axis.title.x=element_blank(),
        axis.title.y=element_text(size=16),
        strip.text.x = element_text(size = 16))
p

ggsave("./HornungOrdinalForests/Results/Figures/FigureSimulationRandomWidthKappa.pdf", width=9*1.2, height=10*1.2)



p <- ggplot(data=results[results$intervals=="equal width",], aes(x=method, y=kappaqu)) + facet_wrap(~ covariates_n + nclass, ncol=3, scales="free_y") + 
  geom_boxplot(width = 0.7) + theme_bw() + ylab("Quadratically weighted Kappa") + 
  theme(axis.text.x=element_text(size=16, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=14, colour="black"),
        axis.title.x=element_blank(),
        axis.title.y=element_text(size=16),
        strip.text.x = element_text(size = 16))
p

ggsave("./HornungOrdinalForests/Results/Figures/FigureSimulationEqualWidthKappaquad.pdf", width=9*1.2, height=10*1.2)


p <- ggplot(data=results[results$intervals=="random width",], aes(x=method, y=kappaqu)) + facet_wrap(~ covariates_n + nclass, ncol=3, scales="free_y") + 
  geom_boxplot(width = 0.7) + theme_bw() + ylab("Quadratically weighted Kappa") + 
  theme(axis.text.x=element_text(size=16, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=14, colour="black"),
        axis.title.x=element_blank(),
        axis.title.y=element_text(size=16),
        strip.text.x = element_text(size = 16))
p

ggsave("./HornungOrdinalForests/Results/Figures/FigureSimulationRandomWidthKappaquad.pdf", width=9*1.2, height=10*1.2)







# Boxplots of AUC values for all settings:
##########################################

p <- ggplot(data=results[results$intervals=="equal width",], aes(x=method, y=auc)) + facet_wrap(~ covariates_n + nclass, ncol=3, scales="free_y") + 
  geom_boxplot(width = 0.7) + theme_bw() + ylab("AUC") + 
  theme(axis.text.x=element_text(size=16, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=14, colour="black"),
        axis.title.x=element_blank(),
        axis.title.y=element_text(size=16),
        strip.text.x = element_text(size = 16))
p

ggsave("./HornungOrdinalForests/Results/Figures/FigureSimulationEqualWidthAUC.pdf", width=9*1.2, height=10*1.2)


p <- ggplot(data=results[results$intervals=="random width",], aes(x=method, y=auc)) + facet_wrap(~ covariates_n + nclass, ncol=3, scales="free_y") + 
  geom_boxplot(width = 0.7) + theme_bw() + ylab("AUC") + 
  theme(axis.text.x=element_text(size=16, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=14, colour="black"),
        axis.title.x=element_blank(),
        axis.title.y=element_text(size=16),
        strip.text.x = element_text(size = 16))
p

ggsave("./HornungOrdinalForests/Results/Figures/FigureSimulationRandomWidthAUC.pdf", width=9*1.2, height=10*1.2)








# Plots of true and estimated class widths for settings with equal class widths:
################################################################################

library("gridExtra")
library("grid")


settings <- c("correlated_n400", "independent_n400", "highdim")
nclasses <- c(3,6,9)

for(setcount in seq(along=settings)) {
  
  for(nclasscount in seq(along=nclasses)) {
    
    
    
    resultstemp <- results[(results$covariates_n %in% c("correlated_n400", "independent_n400", "highdim")) &
                             results$nclass==paste("nclass = ", nclasses[nclasscount], sep="") & results$method=="OF" & results$intervals=="equal width",]
    Resultstemp <- Results[(results$covariates_n %in% c("correlated_n400", "independent_n400", "highdim")) &
                             results$nclass==paste("nclass = ", nclasses[nclasscount], sep="") & results$method=="OF" & results$intervals=="equal width"]
    
    
    
    resultstemp2 <- resultstemp[resultstemp$covariates_n==settings[setcount],]
    Resultstemp2 <- Resultstemp[resultstemp$covariates_n==settings[setcount]]
    
    
    bordersrel <- lapply(Resultstemp2, function(x) x$bordershat)
    
    borders <- matrix(ncol=nclasses[nclasscount]+1, unlist(bordersrel), byrow=TRUE)
    
    
    width <- t(apply(borders, 1, function(x) diff(x)[length(diff(x)):1]))
    
    datatemp1 <- data.frame(x=rep(1:nrow(width), each=ncol(width)), y=as.vector(t(width)), fill=factor(rep(1:ncol(width), times=nrow(width))))
    
    
    borderstemp <- qnorm(borders)
    borderstemp[,1] <- qnorm(0.001)
    borderstemp[,ncol(borderstemp)] <- qnorm(1-0.001)
    width <- t(apply(borderstemp, 1, function(x) diff(x)[length(diff(x)):1]))
    
    datatemp2 <- data.frame(x=rep(1:nrow(width), each=ncol(width)), y=as.vector(t(width)), fill=factor(rep(1:ncol(width), times=nrow(width))))
    
    
    
    p1 <- ggplot(data=datatemp1[1:(20*ncol(width)),], aes(x=x, y=y, fill=fill)) + geom_bar(stat="identity", width=0.7) + 
      theme_bw() + coord_flip() +
      theme(legend.position="none", panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(), panel.border=element_blank(),
            axis.title=element_blank(), title=element_text(size=12),
            axis.text=element_text(size=12, colour="black"))  + scale_y_continuous(expand = c(0.005,0.005), 
                                                                                   breaks=c(0,0.25,0.50,0.75,1), labels=c(0,0.25,0.50,0.75,1)) +  
      scale_x_continuous(expand = c(0.005,0), breaks=c(20,16,11,6,1), labels=c(1,5,10,15,20)) + ggtitle("Partition of [0,1]")
    
    
    p2 <- ggplot(data=datatemp2[1:(20*ncol(width)),], aes(x=x, y=y, fill=fill)) + geom_bar(stat="identity", width=0.7) + 
      theme_bw() + coord_flip() +
      theme(legend.position="none", panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(), panel.border=element_blank(),
            axis.title=element_blank(), title=element_text(size=12),
            axis.text=element_text(size=13, colour="black"))  + scale_y_continuous(expand = c(0.005,0.005), 
                                                                                   breaks=c(-3,-1,1,3) - qnorm(0.001), labels=c(-3,-1,1,3)) +  
      scale_x_continuous(expand = c(0.005,0), breaks=c(20,16,11,6,1), labels=c(1,5,10,15,20)) + ggtitle("Intervals after normal transformation")
    

    
    J <- nclasses[nclasscount]
    
    bordersnorm <- seq(qnorm(0.001), qnorm(1-0.001), length=J+1)
    bordersqu <- c(0,pnorm(bordersnorm[-c(1,length(bordersnorm))]),1)
    
    
    width <- diff(bordersqu)[length(diff(bordersqu)):1]
    datatemp1 <- data.frame(x=1, y=width, fill=factor(1:length(width)))
    
    width <- diff(bordersnorm)[length(diff(bordersnorm)):1]
    datatemp2 <- data.frame(x=1, y=width, fill=factor(1:length(width)))
    
    p3 <- ggplot(data=datatemp1, aes(x=x, y=y, fill=fill)) + geom_bar(stat="identity", width=0.7) + 
      theme_bw() + coord_flip() + 
      theme(legend.position="none", panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(), panel.border=element_blank(),
            axis.title=element_blank(), axis.text.y=element_text(color="white"),
            axis.text=element_text(size=13, colour="black"), 
            axis.ticks.y=element_line(color="white"), title=element_text(size=12))  + 
      scale_y_continuous(expand = c(0.005,0.005), breaks=c(0,0.25,0.50,0.75,1), labels=c(0,0.25,0.50,0.75,1)) +  
      scale_x_continuous(expand = c(0.005,0)) + ggtitle("Partition of [0,1]")
    
    p4 <- ggplot(data=datatemp2, aes(x=x, y=y, fill=fill)) + geom_bar(stat="identity", width=0.7) + 
      theme_bw() + coord_flip() +
      theme(legend.position="none", panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(), panel.border=element_blank(),
            axis.title=element_blank(), axis.text.y=element_text(color="white"),
            axis.text=element_text(size=13, colour="black"), 
            axis.ticks.y=element_line(color="white"), title=element_text(size=12))  + 
      scale_y_continuous(expand = c(0.005,0.005), breaks=c(-3,-1,1,3) - qnorm(0.001), labels=c(-3,-1,1,3)) +  
      scale_x_continuous(expand = c(0.005,0)) + ggtitle("Intervals after normal transformation")
    
    
    grobframe1 <- arrangeGrob(p1, p2, ncol=2, nrow=1, 
                              top=textGrob("Estimated class widths for 20 simulated datasets", gp=gpar(fontsize=17)))
    grobframe1 
    
    grobframe2 <- arrangeGrob(p3, p4, ncol=2, top=textGrob("True class widths", gp=gpar(fontsize=17)))
    grobframe2 
    
    pall <- grid.arrange(grobframe2, grobframe1, nrow = 2, heights=c(1,6))
    
    
    ggsave(file=paste("./HornungOrdinalForests/Results/Figures/FigurePartitionsEqualClasswidth", settings[setcount], 
                      "nclass", nclasses[nclasscount], ".pdf", sep=""), plot=pall, width=9*1.2, height=10*1.2)
    
  }
  
}










# Plots of true and estimated class widths for settings with random class widths:
##################################################################################

settings <- c("correlated_n400", "independent_n400", "highdim")
nclasses <- c(3,6,9)


for(setcount in seq(along=settings)) {
  
  for(nclasscount in seq(along=nclasses)) {
    
    resultstemp <- results[(results$covariates_n %in% c("correlated_n400", "independent_n400", "highdim")) &
                             results$nclass==paste("nclass = ", nclasses[nclasscount], sep="") & results$method=="OF" & results$intervals=="random width",]
    Resultstemp <- Results[(results$covariates_n %in% c("correlated_n400", "independent_n400", "highdim")) &
                             results$nclass==paste("nclass = ", nclasses[nclasscount], sep="") & results$method=="OF" & results$intervals=="random width"]
    
    resultstemp2 <- resultstemp[resultstemp$covariates_n==settings[setcount],]
    Resultstemp2 <- Resultstemp[resultstemp$covariates_n==settings[setcount]]
    
    
    borderstrue <- lapply(Resultstemp2, function(x) x$borderstrue)
    bordershat <- lapply(Resultstemp2, function(x) x$bordershat)
    
    borderstrue <- matrix(ncol=length(borderstrue[[1]]), unlist(borderstrue), byrow=TRUE)
    bordershat <- matrix(ncol=length(bordershat[[1]]), unlist(bordershat), byrow=TRUE)
    
    
    bordersnormtrue <- qnorm(borderstrue)
    bordersnormtrue[,1] <- qnorm(0.001)
    bordersnormtrue[,ncol(bordersnormtrue)] <- qnorm(1-0.001)
    
    bordersnormhat <- qnorm(bordershat)
    bordersnormhat[,1] <- qnorm(0.001)
    bordersnormhat[,ncol(bordersnormhat)] <- qnorm(1-0.001)
    
    
    
    borderstrue <- split(t(borderstrue), rep(1:nrow(borderstrue), each = ncol(borderstrue)))
    bordershat <- split(t(bordershat), rep(1:nrow(bordershat), each = ncol(bordershat)))
    
    bordersnormtrue <- split(t(bordersnormtrue), rep(1:nrow(bordersnormtrue), each = ncol(bordersnormtrue)))
    bordersnormhat <- split(t(bordersnormhat), rep(1:nrow(bordersnormhat), each = ncol(bordersnormhat)))
    
    
    gg_color_hue <- function(n) {
      hues = seq(15, 375, length = n + 1)
      hcl(h = hues, l = 65, c = 100)[1:n]
    }
    colorstemp <- gg_color_hue(nclasses[nclasscount])
    colorstemp <- colorstemp[length(colorstemp):1]
    
    
    colorstrue <- lapply(borderstrue, function(x) {
      if(any(diff(x) < 0))
        return(colorstemp[-(length(colorstemp) - which(diff(x) < 0) + 1)])
      else
        return(colorstemp)
    })
    
    colorshat <- lapply(bordershat, function(x) {
      if(any(diff(x) < 0))
        return(colorstemp[-(length(colorstemp) - which(diff(x) < 0) + 1)])
      else
        return(colorstemp)
    })
    
    
    colorsnormtrue <- lapply(bordersnormtrue, function(x) {
      if(any(diff(x) < 0))
        return(colorstemp[-(length(colorstemp) - which(diff(x) < 0) + 1)])
      else
        return(colorstemp)
    })
    
    colorsnormhat <- lapply(bordersnormhat, function(x) {
      if(any(diff(x) < 0))
        return(colorstemp[-(length(colorstemp) - which(diff(x) < 0) + 1)])
      else
        return(colorstemp)
    })
    
    bordersnormtrue <- lapply(bordersnormtrue, function(x) x[x>=qnorm(0.001) & x<=qnorm(1-0.001)])
    bordersnormhat <- lapply(bordersnormhat, function(x) x[x>=qnorm(0.001) & x<=qnorm(1-0.001)])
    
    
    widthtrue <- lapply(borderstrue, function(x) {
      res <- diff(x)
      res[length(res):1]
    })
    widthhat <-lapply(bordershat, function(x) {
      res <- diff(x)
      res[length(res):1]
    })
    
    widthnormtrue <- lapply(bordersnormtrue, function(x) {
      res <- diff(x)
      res[length(res):1]
    })
    widthnormhat <- lapply(bordersnormhat, function(x) {
      res <- diff(x)
      res[length(res):1]
    })
    
    datatemptrue <- data.frame(x=rep(1:length(widthtrue), times=sapply(widthtrue, length)), y=unlist(widthtrue), fill=factor(unlist(colorstrue), levels=colorstemp))
    datatemphat <- data.frame(x=rep(1:length(widthhat), times=sapply(widthhat, length)), y=unlist(widthhat), fill=factor(unlist(colorshat), levels=colorstemp))
    
    datatempnormtrue <- data.frame(x=rep(1:length(widthnormtrue), times=sapply(widthnormtrue, length)), y=unlist(widthnormtrue), fill=factor(unlist(colorsnormtrue), levels=colorstemp))
    datatempnormhat <- data.frame(x=rep(1:length(widthnormhat), times=sapply(widthnormhat, length)), y=unlist(widthnormhat), fill=factor(unlist(colorsnormhat), levels=colorstemp))
    
    
    
    grobframes <- list()
    
    for(i in 1:10) {
      
      datatemp <- rbind(datatemphat[datatemphat$x==i,], datatemptrue[datatemptrue$x==i,])
      datatemp$x <- rep(c(1, 2), times=c(nrow(datatemptrue[datatemptrue$x==i,]), nrow(datatemphat[datatemphat$x==i,])))
      
      datatempnorm <- rbind(datatempnormhat[datatempnormhat$x==i,], datatempnormtrue[datatempnormtrue$x==i,])
      datatempnorm$x <- rep(c(1, 2), times=c(nrow(datatempnormhat[datatempnormhat$x==i,]), nrow(datatempnormtrue[datatempnormtrue$x==i,])))
      
      p1 <- ggplot(data=datatemp, aes(x=x, y=y, fill=fill)) + geom_bar(stat="identity") + 
        theme_bw() + coord_flip() +
        theme(legend.position="none", panel.grid.major = element_blank(),
              panel.grid.minor = element_blank(), panel.border=element_blank(),
              axis.title=element_blank(), title=element_text(size=13),
              axis.text=element_text(size=13, colour="black"))  + scale_y_continuous(expand = c(0.005,0.005), breaks=c(0,0.25,0.50,0.75,1), labels=c(0,0.25,0.50,0.75,1)) +  
        scale_x_continuous(expand = c(0.005,0), breaks=c(1,2), labels=c("estimated", "true")) + ggtitle("Partition of [0,1]")
      
      p2 <- ggplot(data=datatempnorm, aes(x=x, y=y, fill=fill)) + geom_bar(stat="identity") +
        theme_bw() + coord_flip() + # facet_wrap(~type, ncol=2, scales="free_x") +
        theme(legend.position="none", panel.grid.major = element_blank(),
              panel.grid.minor = element_blank(), panel.border=element_blank(),
              axis.title=element_blank(), title=element_text(size=13),
              axis.text=element_text(size=13, colour="black"))  + scale_y_continuous(expand = c(0.005,0.005), breaks=c(-3,-1,1,3) - qnorm(0.001), labels=c(-3,-1,1,3)) +  
        scale_x_continuous(expand = c(0.005,0), breaks=c(1,2), labels=c("estimated", "true")) + ggtitle("Intervals after normal transformation")
      
      grobframes[[i]] <- arrangeGrob(p1, p2, ncol=2, nrow=1)
      
    } 
    
    pall <- grid.arrange(grobframes[[1]], grobframes[[2]], grobframes[[3]], 
                         grobframes[[4]], grobframes[[5]], grobframes[[6]], 
                         grobframes[[7]], grobframes[[8]], grobframes[[9]], 
                         grobframes[[10]], nrow = 10)
    
    ggsave(file=paste("./HornungOrdinalForests/Results/Figures/FigurePartitionsRandomClasswidth",
                      settings[setcount], "nclass", nclasses[nclasscount], ".pdf", sep=""), plot=pall, width=9*1.2, height=10*1.2)
    
  }
}










# Scatter plots of estimated versus true class widths for settings with random class widths:
############################################################################################


# setting 'correlated_n400':

resultstemp <- results[results$covariates_n=="correlated_n400" &
                         results$method=="OF" & results$intervals=="random width",]
Resultstemp <- Results[results$covariates_n=="correlated_n400" &
                         results$method=="OF" & results$intervals=="random width"]


bordersrel <- lapply(Resultstemp, function(x) cbind(x$borderstrue, x$bordershat))
allwidths <- lapply(bordersrel, function(x) apply(x, 2, diff))


wd <- allwidths[which(sapply(bordersrel, function(x) nrow(x)==3+1))]

datatemp1 <- data.frame(do.call("rbind", wd))
datatemp1$classnum <- rep(1:nrow(wd[[1]]), times=length(wd))

datatemp1$class <- "other class"
datatemp1$class[datatemp1$classnum %in% c(1, nrow(wd[[1]]))] <- paste("1 or ", nrow(wd[[1]]), sep="")
datatemp1$class <- factor(datatemp1$class, levels=c(paste("1 or ", nrow(wd[[1]]), sep=""), "other class"))



wd <- allwidths[which(sapply(bordersrel, function(x) nrow(x)==6+1))]

datatemp2 <- data.frame(do.call("rbind", wd))
datatemp2$classnum <- rep(1:nrow(wd[[1]]), times=length(wd))

datatemp2$class <- "other class"
datatemp2$class[datatemp2$classnum %in% c(1, nrow(wd[[1]]))] <- paste("1 or ", nrow(wd[[1]]), sep="")
datatemp2$class[datatemp2$classnum %in% c(2, nrow(wd[[1]])-1)] <- paste("2 or ", nrow(wd[[1]])-1, sep="")

datatemp2$class <- factor(datatemp2$class, levels=c(paste("1 or ", nrow(wd[[1]]), sep=""), paste("2 or ", nrow(wd[[1]])-1, sep=""), "other class"))



wd <- allwidths[which(sapply(bordersrel, function(x) nrow(x)==9+1))]

datatemp3 <- data.frame(do.call("rbind", wd))
datatemp3$classnum <- rep(1:nrow(wd[[1]]), times=length(wd))

datatemp3$class <- "other class"
datatemp3$class[datatemp3$classnum %in% c(1, nrow(wd[[1]]))] <- paste("1 or ", nrow(wd[[1]]), sep="")
datatemp3$class[datatemp3$classnum %in% c(2, nrow(wd[[1]])-1)] <- paste("2 or ", nrow(wd[[1]])-1, sep="")

datatemp3$class <- factor(datatemp3$class, levels=c(paste("1 or ", nrow(wd[[1]]), sep=""), paste("2 or ", nrow(wd[[1]])-1, sep=""), "other class"))


set.seed(1234)

ds <- datatemp1

p1 <- ggplot(data=ds) + stat_bin2d(aes(x=X1, y=X2), bins = 10) + geom_point(data=ds[sample(1:nrow(ds), size=200),], aes(x=X1, y=X2, color=class), size=0.8) + 
   stat_smooth(aes(x=X1, y=X2, color=class), formula = y ~ s(x), method="gam", method.args = list(family=quasibinomial(link = "logit")), se=FALSE) +
    theme_bw() + expand_limits(y=1) + labs(x="True class width", y="Estimated class width") + ggtitle("nclass = 3") + 
  theme(plot.title = element_text(size=18),
                                                                                                                            axis.title=element_text(size=15),
                                                                                                                            axis.text=element_text(size=12),
                                                                                                                            legend.title = element_text(size=15),
                                                                                                                            legend.text = element_text(size=15),
                                                                                                                            plot.margin=unit(c(0.2,0,0.2,0), "mm"))

ds <- datatemp2

p2 <- ggplot(data=ds) + stat_bin2d(aes(x=X1, y=X2), bins = 10) + geom_point(data=ds[sample(1:nrow(ds), size=200),], aes(x=X1, y=X2, color=class), size=0.8) + 
  stat_smooth(aes(x=X1, y=X2, color=class), formula = y ~ s(x), method="gam", method.args = list(family=quasibinomial(link = "logit")), se=FALSE) +
  theme_bw() + expand_limits(y=1) + labs(x="True class width", y="Estimated class width") + ggtitle("nclass = 6") + 
  theme(plot.title = element_text(size=18),
                                                                                                                          axis.title=element_text(size=15),
                                                                                                                          axis.text=element_text(size=12),
                                                                                                                          legend.title = element_text(size=15),
                                                                                                                          legend.text = element_text(size=15),
                                                                                                                          plot.margin=unit(c(0.2,0,0.2,0), "mm"))

ds <- datatemp3

p3 <- ggplot(data=ds) + stat_bin2d(aes(x=X1, y=X2), bins = 10) + geom_point(data=ds[sample(1:nrow(ds), size=200),], aes(x=X1, y=X2, color=class), size=0.8) + 
  stat_smooth(aes(x=X1, y=X2, color=class), formula = y ~ s(x), method="gam", method.args = list(family=quasibinomial(link = "logit")), se=FALSE) +
  theme_bw() + expand_limits(y=1) + labs(x="True class width", y="Estimated class width") + 
  ggtitle("nclass = 9") + theme(plot.title = element_text(size=18),
                                axis.title=element_text(size=15),
                                axis.text=element_text(size=12),
                                legend.title = element_text(size=15),
                                legend.text = element_text(size=15),
                                plot.margin=unit(c(0.2,0,0.2,0), "mm"))

pall <- grid.arrange(p1, p2, p3, nrow = 1)

ggsave(file="./HornungOrdinalForests/Results/Figures/FigureTrueVsEstRandomClasswidthcorrelated_n400.pdf", plot=pall, width=16*0.8, height=4*0.8)





# setting 'independent_n400':


resultstemp <- results[results$covariates_n=="independent_n400" &
                         results$method=="OF" & results$intervals=="random width",]
Resultstemp <- Results[results$covariates_n=="independent_n400" &
                         results$method=="OF" & results$intervals=="random width"]


bordersrel <- lapply(Resultstemp, function(x) cbind(x$borderstrue, x$bordershat))
allwidths <- lapply(bordersrel, function(x) apply(x, 2, diff))


wd <- allwidths[which(sapply(bordersrel, function(x) nrow(x)==3+1))]

datatemp1 <- data.frame(do.call("rbind", wd))
datatemp1$classnum <- rep(1:nrow(wd[[1]]), times=length(wd))

datatemp1$class <- "other class"
datatemp1$class[datatemp1$classnum %in% c(1, nrow(wd[[1]]))] <- paste("1 or ", nrow(wd[[1]]), sep="")
datatemp1$class <- factor(datatemp1$class, levels=c(paste("1 or ", nrow(wd[[1]]), sep=""), "other class"))



wd <- allwidths[which(sapply(bordersrel, function(x) nrow(x)==6+1))]

datatemp2 <- data.frame(do.call("rbind", wd))
datatemp2$classnum <- rep(1:nrow(wd[[1]]), times=length(wd))

datatemp2$class <- "other class"
datatemp2$class[datatemp2$classnum %in% c(1, nrow(wd[[1]]))] <- paste("1 or ", nrow(wd[[1]]), sep="")
datatemp2$class[datatemp2$classnum %in% c(2, nrow(wd[[1]])-1)] <- paste("2 or ", nrow(wd[[1]])-1, sep="")

datatemp2$class <- factor(datatemp2$class, levels=c(paste("1 or ", nrow(wd[[1]]), sep=""), paste("2 or ", nrow(wd[[1]])-1, sep=""), "other class"))



wd <- allwidths[which(sapply(bordersrel, function(x) nrow(x)==9+1))]

datatemp3 <- data.frame(do.call("rbind", wd))
datatemp3$classnum <- rep(1:nrow(wd[[1]]), times=length(wd))

datatemp3$class <- "other class"
datatemp3$class[datatemp3$classnum %in% c(1, nrow(wd[[1]]))] <- paste("1 or ", nrow(wd[[1]]), sep="")
datatemp3$class[datatemp3$classnum %in% c(2, nrow(wd[[1]])-1)] <- paste("2 or ", nrow(wd[[1]])-1, sep="")

datatemp3$class <- factor(datatemp3$class, levels=c(paste("1 or ", nrow(wd[[1]]), sep=""), paste("2 or ", nrow(wd[[1]])-1, sep=""), "other class"))




set.seed(1234)

ds <- datatemp1

p1 <- ggplot(data=ds) + stat_bin2d(aes(x=X1, y=X2), bins = 10) + geom_point(data=ds[sample(1:nrow(ds), size=200),], aes(x=X1, y=X2, color=class), size=0.8) + 
  stat_smooth(aes(x=X1, y=X2, color=class), formula = y ~ s(x), method="gam", method.args = list(family=quasibinomial(link = "logit")), se=FALSE) +
  theme_bw() + expand_limits(y=1) + labs(x="True class width", y="Estimated class width") + ggtitle("nclass = 3") + 
  theme(plot.title = element_text(size=18),
        axis.title=element_text(size=15),
        axis.text=element_text(size=12),
        legend.title = element_text(size=15),
        legend.text = element_text(size=15),
        plot.margin=unit(c(0.2,0,0.2,0), "mm"))

ds <- datatemp2

p2 <- ggplot(data=ds) + stat_bin2d(aes(x=X1, y=X2), bins = 10) + geom_point(data=ds[sample(1:nrow(ds), size=200),], aes(x=X1, y=X2, color=class), size=0.8) + 
  stat_smooth(aes(x=X1, y=X2, color=class), formula = y ~ s(x), method="gam", method.args = list(family=quasibinomial(link = "logit")), se=FALSE) +
  theme_bw() + expand_limits(y=1) + labs(x="True class width", y="Estimated class width") + ggtitle("nclass = 6") + 
  theme(plot.title = element_text(size=18),
        axis.title=element_text(size=15),
        axis.text=element_text(size=12),
        legend.title = element_text(size=15),
        legend.text = element_text(size=15),
        plot.margin=unit(c(0.2,0,0.2,0), "mm"))

ds <- datatemp3

p3 <- ggplot(data=ds) + stat_bin2d(aes(x=X1, y=X2), bins = 10) + geom_point(data=ds[sample(1:nrow(ds), size=200),], aes(x=X1, y=X2, color=class), size=0.8) + 
  stat_smooth(aes(x=X1, y=X2, color=class), formula = y ~ s(x), method="gam", method.args = list(family=quasibinomial(link = "logit")), se=FALSE) +
  theme_bw() + expand_limits(y=1) + labs(x="True class width", y="Estimated class width") + ggtitle("nclass = 9") + 
  theme(plot.title = element_text(size=18),
        axis.title=element_text(size=15),
        axis.text=element_text(size=12),
        legend.title = element_text(size=15),
        legend.text = element_text(size=15),
        plot.margin=unit(c(0.2,0,0.2,0), "mm"))

pall <- grid.arrange(p1, p2, p3, nrow = 1)

ggsave(file="./HornungOrdinalForests/Results/Figures/FigureTrueVsEstRandomClasswidthindependent_n400.pdf", plot=pall, width=16*0.8, height=4*0.8)




# setting 'highdim':


resultstemp <- results[results$covariates_n=="highdim" &
                         results$method=="OF" & results$intervals=="random width",]
Resultstemp <- Results[results$covariates_n=="highdim" &
                         results$method=="OF" & results$intervals=="random width"]


bordersrel <- lapply(Resultstemp, function(x) cbind(x$borderstrue, x$bordershat))
allwidths <- lapply(bordersrel, function(x) apply(x, 2, diff))


wd <- allwidths[which(sapply(bordersrel, function(x) nrow(x)==3+1))]

datatemp1 <- data.frame(do.call("rbind", wd))
datatemp1$classnum <- rep(1:nrow(wd[[1]]), times=length(wd))

datatemp1$class <- "other class"
datatemp1$class[datatemp1$classnum %in% c(1, nrow(wd[[1]]))] <- paste("1 or ", nrow(wd[[1]]), sep="")
datatemp1$class <- factor(datatemp1$class, levels=c(paste("1 or ", nrow(wd[[1]]), sep=""), "other class"))



wd <- allwidths[which(sapply(bordersrel, function(x) nrow(x)==6+1))]

datatemp2 <- data.frame(do.call("rbind", wd))
datatemp2$classnum <- rep(1:nrow(wd[[1]]), times=length(wd))

datatemp2$class <- "other class"
datatemp2$class[datatemp2$classnum %in% c(1, nrow(wd[[1]]))] <- paste("1 or ", nrow(wd[[1]]), sep="")
datatemp2$class[datatemp2$classnum %in% c(2, nrow(wd[[1]])-1)] <- paste("2 or ", nrow(wd[[1]])-1, sep="")

datatemp2$class <- factor(datatemp2$class, levels=c(paste("1 or ", nrow(wd[[1]]), sep=""), paste("2 or ", nrow(wd[[1]])-1, sep=""), "other class"))



wd <- allwidths[which(sapply(bordersrel, function(x) nrow(x)==9+1))]

datatemp3 <- data.frame(do.call("rbind", wd))
datatemp3$classnum <- rep(1:nrow(wd[[1]]), times=length(wd))

datatemp3$class <- "other class"
datatemp3$class[datatemp3$classnum %in% c(1, nrow(wd[[1]]))] <- paste("1 or ", nrow(wd[[1]]), sep="")
datatemp3$class[datatemp3$classnum %in% c(2, nrow(wd[[1]])-1)] <- paste("2 or ", nrow(wd[[1]])-1, sep="")

datatemp3$class <- factor(datatemp3$class, levels=c(paste("1 or ", nrow(wd[[1]]), sep=""), paste("2 or ", nrow(wd[[1]])-1, sep=""), "other class"))



set.seed(1234)

ds <- datatemp1

p1 <- ggplot(data=ds) + stat_bin2d(aes(x=X1, y=X2), bins = 10) + geom_point(data=ds[sample(1:nrow(ds), size=200),], aes(x=X1, y=X2, color=class), size=0.8) + 
  stat_smooth(aes(x=X1, y=X2, color=class), formula = y ~ s(x), method="gam", method.args = list(family=quasibinomial(link = "logit")), se=FALSE) +
  theme_bw() + expand_limits(y=1) + labs(x="True class width", y="Estimated class width") + ggtitle("nclass = 3") + 
  theme(plot.title = element_text(size=18),
        axis.title=element_text(size=15),
        axis.text=element_text(size=12),
        legend.title = element_text(size=15),
        legend.text = element_text(size=15),
        plot.margin=unit(c(0.2,0,0.2,0), "mm"))

ds <- datatemp2

p2 <- ggplot(data=ds) + stat_bin2d(aes(x=X1, y=X2), bins = 10) + geom_point(data=ds[sample(1:nrow(ds), size=200),], aes(x=X1, y=X2, color=class), size=0.8) + 
  stat_smooth(aes(x=X1, y=X2, color=class), formula = y ~ s(x), method="gam", method.args = list(family=quasibinomial(link = "logit")), se=FALSE) +
  theme_bw() + expand_limits(y=1) + labs(x="True class width", y="Estimated class width") + ggtitle("nclass = 6") + 
  theme(plot.title = element_text(size=18),
        axis.title=element_text(size=15),
        axis.text=element_text(size=12),
        legend.title = element_text(size=15),
        legend.text = element_text(size=15),
        plot.margin=unit(c(0.2,0,0.2,0), "mm"))

ds <- datatemp3

p3 <- ggplot(data=ds) + stat_bin2d(aes(x=X1, y=X2), bins = 10) + geom_point(data=ds[sample(1:nrow(ds), size=200),], aes(x=X1, y=X2, color=class), size=0.8) + 
  stat_smooth(aes(x=X1, y=X2, color=class), formula = y ~ s(x), method="gam", method.args = list(family=quasibinomial(link = "logit")), se=FALSE) +
  theme_bw() + expand_limits(y=1) + labs(x="True class width", y="Estimated class width") + ggtitle("nclass = 9") + 
  theme(plot.title = element_text(size=18),
        axis.title=element_text(size=15),
        axis.text=element_text(size=12),
        legend.title = element_text(size=15),
        legend.text = element_text(size=15),
        plot.margin=unit(c(0.2,0,0.2,0), "mm"))

pall <- grid.arrange(p1, p2, p3, nrow = 1)

ggsave(file="./HornungOrdinalForests/Results/Figures/FigureTrueVsEstRandomClasswidthhighdim.pdf", plot=pall, width=16*0.8, height=4*0.8)











# Study for which kind partitions there is a stronger improvement through the optimization of the
# scores performed by OF, that is, for which there is a particularly strong improvement of OF over
# naive OF:
###################################################################################################


# Subset results to feature only methods OF, naive OF and interval type "random width":
resultsbor <- results[results$method %in% c("OF", "naive OF") & results$intervals=="random width",]
Resultsbor <- Results[results$method %in% c("OF", "naive OF") & results$intervals=="random width"]


# Resort results so that the pairs of results obtained for naive OF and OF stand together:
orderind <- order(resultsbor$covariates_n, resultsbor$nclass, resultsbor$simit, resultsbor$method)

resultsbor <- resultsbor[orderind,]
Resultsbor <- Resultsbor[orderind]

head(resultsbor)


# Exclude pairs of results for which at least one of the linearly weighted Kappa
# values is smaller than 0.1:
inds <- rep(1:(nrow(resultsbor)/2), each=2)
indsun <- unique(inds)
inclbool <- sapply(indsun, function(x) all(resultsbor$kappalin[inds==x] > 0.10))
indrel <- indsun[inclbool]

inclind <- which(inds %in% indrel)

resultsbor <- resultsbor[inclind,]
Resultsbor <- Resultsbor[inclind]


# For each pair calculate the ratio between the result obtained for OF
# and that obtained for naive OF:
indrel <- seq(2, nrow(resultsbor), by=2)
resultstemp <- resultsbor[indrel,]
Resultstemp <- Resultsbor[indrel]

resultstemp$kappalinratio <- resultsbor$kappalin[seq(2, nrow(resultsbor), by=2)]/resultsbor$kappalin[seq(1, nrow(resultsbor), by=2)]


# Only include the settings for the larger sample size (n=400) because
# here more stable results can be expected to be obtained:
exclind <- grep("n200", resultstemp$covariates_n)

resultstemp <- resultstemp[-exclind,]
Resultstemp <- Resultstemp[-exclind]




# Table of settings:
scens <- unique(resultstemp[,1:2])
scens

# List of booleans which give the indices of the 
# results for each setting:
scensindlist <- lapply(1:nrow(scens), function(x) apply(resultstemp[,1:2], 1, function(y) all(y==scens[x,])))


# For each setting extract result table and true borders
# and order decreasingly by the ratio between linear Kappa of
# OF and naive OF:
resratio <- lapply(scensindlist, function(x) {
  
  resultst <- resultstemp[x,]
  borderst <- lapply(Resultstemp[x], function(y) y$borderstrue)
  
  orderind <- order(resultst$kappalinratio, decreasing=FALSE)
  
  resultst <- resultst[orderind,]
  borderst <- borderst[orderind]
  
  return(list(resultst=resultst, borderst=borderst))
  
})


# Number of results for each setting (with lin. Kappa > 0.1 for both OF and naive OF):
sapply(1:nrow(scens), function(count) length(resratio[[count]]$borderst))




# Make plot of true partitions of [0,1] and corresponding ratio between linearly weighted
# Kappa obtained for OF and naive OF:


# First setting:

count <- 1

# Borders in first setting:
resmat <- matrix(nrow=length(resratio[[count]]$borderst), ncol=length(resratio[[count]]$borderst[[1]]), data=unlist(resratio[[count]]$borderst), byrow=TRUE)

datatemp <- data.frame(x=rep(1:nrow(resmat), each=ncol(resmat)-1), fill=factor(rep(1:(ncol(resmat)-1))), y=as.vector(apply(-resmat[,ncol(resmat):1], 1, diff)))

p1 <- ggplot(data=datatemp, aes(x=x, y=y, fill=fill)) + geom_bar(stat="identity", width=0.7) + theme_bw()+ coord_flip() +
  theme(legend.position="none", panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(), panel.border=element_blank(),
        axis.title=element_blank(),
        title=element_text(size=11),
        axis.text=element_text(size=9, colour="black")) + scale_y_continuous(expand = c(0.005,0.005)) +
  scale_x_continuous(expand = c(0.005,0)) + ggtitle("True partition\n\n")
p1

datatemp2 <- data.frame(x=resratio[[count]]$resultst$kappalinratio, y=1:length(resratio[[count]]$borderst))

p2 <- ggplot(data=datatemp2, aes(x=x, y=y)) + geom_point() + theme_bw() + 
  theme(panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(), panel.border=element_blank(),
        axis.title=element_blank(), axis.text.y=element_blank(), 
        axis.line.x = element_line(colour = "black"), axis.ticks.y=element_blank(),
        title=element_text(size=11),
        axis.text=element_text(size=9, colour="black")) + scale_y_continuous(expand = c(0.005,0)) + 
  scale_x_continuous(expand = c(0.02,0)) + geom_vline(xintercept=1) + ggtitle("Ratio between linearly\n weighted Kappa obtained\n for OF and naive OF")
p2

grid.arrange(p1, p2, ncol = 2, nrow = 1, widths = c(4,1))

p <- arrangeGrob(p1, p2, ncol = 2, nrow = 1, widths = c(4,1.9))

ggsave(paste("./HornungOrdinalForests/Results/Figures/FigurePartition", 
             gsub(" = ", "", paste(sapply(scens[count,], as.character), collapse="")), ".pdf", sep=""), plot=p, width=8, height=10)


for(count in 2:nrow(scens)) {

  resmat <- matrix(nrow=length(resratio[[count]]$borderst), ncol=length(resratio[[count]]$borderst[[1]]), data=unlist(resratio[[count]]$borderst), byrow=TRUE)
  
  datatemp <- data.frame(x=rep(1:nrow(resmat), each=ncol(resmat)-1), fill=factor(rep(1:(ncol(resmat)-1))), y=as.vector(apply(-resmat[,ncol(resmat):1], 1, diff)))
  
  p1 <- ggplot(data=datatemp, aes(x=x, y=y, fill=fill)) + geom_bar(stat="identity", width=0.7) + theme_bw()+ coord_flip() +
    theme(legend.position="none", panel.grid.major = element_blank(),
          panel.grid.minor = element_blank(), panel.border=element_blank(),
          axis.title=element_blank(),
          title=element_text(size=11),
          axis.text=element_text(size=9, colour="black")) + scale_y_continuous(expand = c(0.005,0.005)) +
    scale_x_continuous(expand = c(0.005,0)) + ggtitle("True partition\n\n")
  p1
  
  
  datatemp2 <- data.frame(x=resratio[[count]]$resultst$kappalinratio, y=1:length(resratio[[count]]$borderst))

  p2 <- ggplot(data=datatemp2, aes(x=x, y=y)) + geom_point() + theme_bw() + 
    theme(panel.grid.major = element_blank(),
          panel.grid.minor = element_blank(), panel.border=element_blank(),
          axis.title=element_blank(), axis.text.y=element_blank(), 
          axis.line.x = element_line(colour = "black"), axis.ticks.y=element_blank(),
          title=element_text(size=11),
          axis.text=element_text(size=9, colour="black")) + scale_y_continuous(expand = c(0.005,0)) + 
    scale_x_continuous(expand = c(0.02,0)) + geom_vline(xintercept=1) + ggtitle("Ratio between linearly\n weighted Kappa obtained\n for OF and naive OF")
  p2
  
  grid.arrange(p1, p2, ncol = 2, nrow = 1, widths = c(4,1))
  
  p <- arrangeGrob(p1, p2, ncol = 2, nrow = 1, widths = c(4,1.9))
  
  ggsave(paste("./HornungOrdinalForests/Results/Figures/FigurePartition", 
               gsub(" = ", "", paste(sapply(scens[count,], as.character), collapse="")), ".pdf", sep=""), plot=p, width=8, height=10)
  
}
